@@ -19,7 +19,33 @@ class DataAccess:
         self.userID = userID
         self.url = url
 
-    def get_sensor_alias(self, device_id, df, raw_metadata):
+    def get_device_metadata(self, device_id):
+        """
+
+        :param device_id: string
+        :return: Json
+
+         Every detail related to a particular device like device added date, m,c,min,max values, sensor id, sensor alias and so on
+
+        """
+        try:
+            url = "https://" + self.url + "/api/metaData/device/" + device_id
+            header = {'userID': self.userID}
+            payload = {}
+            response = requests.request('GET', url, headers=header, data=payload, verify=False)
+            # print(response.text)
+            if response.status_code != 200:
+                raw = json.loads(response.text)
+                raise ValueError(raw['error'])
+            else:
+                raw_data = json.loads(response.text)['data']
+                return raw_data
+
+        except Exception as e:
+            print('Failed to fetch device Metadata')
+            print(e)
+
+    def get_sensor_alias(self, device_id, df,raw_metadata):
         """
 
         :param raw_metadata: json of device metadata
@@ -37,14 +63,17 @@ class DataAccess:
             raw_metadata = DataAccess.get_device_metadata(self, device_id)
         sensor_spec = 'sensors'
         sensor_param_df = pd.DataFrame(raw_metadata[sensor_spec])
+
         for i in list1:
             sensor_param_df1 = sensor_param_df[sensor_param_df['sensorId'] == i]
-            sensor_name = sensor_param_df1.iloc[0]['sensorName']
-            sensor_name = sensor_name + " (" + i + ")"
-            df['sensor'] = df['sensor'].replace(i, sensor_name)
-        return df
+            if len(sensor_param_df1) != 0:
+                # print('sensor_param_df',sensor_param_df1)
+                sensor_name = sensor_param_df1.iloc[0]['sensorName']
+                sensor_name = sensor_name + " (" + i + ")"
+                df['sensor'] = df['sensor'].replace(i, sensor_name)
+        return df,raw_metadata
 
-    def get_caliberation(self, device_id, df_raw):
+    def get_caliberation(self,device_id,metadata,df):
         """
 
         :param df_raw: Dataframe
@@ -57,67 +86,36 @@ class DataAccess:
              if y is less than min value replace y with min value
 
         """
+        sensor_name_list = list(df.columns)
+        sensor_name_list.remove('time')
+        # print(sensor_name_list)
+        sensor_id_list = [s[s.rfind("(") + 1:s.rfind(")")] for s in sensor_name_list]
+        if len(metadata) == 0:
+            metadata = DataAccess.get_device_metadata(self,device_id)
+        data = metadata['params']
 
-        df_raw.sort_values("sensor", inplace=True)
-        raw_metadata = DataAccess.get_device_metadata(self, device_id)
-        sensor_spec = 'params'
-        header = ['sensorID', 'm', 'c', 'min', 'max', 'automation']
-        sen_spec_data = raw_metadata[sensor_spec].keys()
-        sen_spec_data_collect = []
-        for sensor in sen_spec_data:
-            m = c = min_ = max_ = automation = 0
-            for name_value in raw_metadata[sensor_spec][sensor]:
-                if name_value['paramName'] == 'm':
-                    m = name_value['paramValue']
-                if name_value['paramName'] == 'c':
-                    c = name_value['paramValue']
-                if name_value['paramName'] == 'min':
-                    min_ = name_value['paramValue']
-                if name_value['paramName'] == 'max':
-                    max_ = name_value['paramValue']
-                if name_value['paramName'] == 'automation':
-                    automation = name_value['paramValue']
-            sen_spec_data_collect.append([sensor, m, c, min_, max_, automation])
-        sensor_param_df = pd.DataFrame(sen_spec_data_collect, columns=header)
-
-        sensor_data_with_meta = df_raw.merge(sensor_param_df, left_on='sensor', right_on='sensorID').drop('sensor',
-                                                                                                          axis=1)
-        if sensor_data_with_meta['c'].values[0] == 0 and sensor_data_with_meta["max"].values[0] == 0 and \
-                sensor_data_with_meta["min"].values[0] == 0:
-            sensor_data_with_meta['final_value'] = sensor_data_with_meta["value"]
-        else:
-            # print(sensor_data_with_meta["value"])
-            sensor_data_with_meta["value"] = sensor_data_with_meta["value"].replace('BAD 255', '-99999') \
-                .replace('-', '99999').replace('BAD undefined', '-99999').replace('BAD 0', '-99999')
-            sensor_data_with_meta["value"] = sensor_data_with_meta["value"].astype('float')
-            sensor_data_with_meta["m"] = sensor_data_with_meta["m"].astype('float')
-            sensor_data_with_meta["c"] = sensor_data_with_meta["c"].astype('int')
-            sensor_data_with_meta["max"] = sensor_data_with_meta["max"].replace('-', '99999').replace(
-                '1000000000000000000000000000', '99999').replace('100000000000', '99999')
-            if type(sensor_data_with_meta["max"][0]) != int:
-                sensor_data_with_meta["max"] = sensor_data_with_meta["max"].astype(float)
-            sensor_data_with_meta["min"] = sensor_data_with_meta["min"].astype('int')
-            # print(sensor_data_with_meta["value"],type((sensor_data_with_meta["value"].values[0])))
-            # abssensor_data_with_meta["value"] = sensor_data_with_meta["value"].astype('str')
-            sensor_data_with_meta["value"] = sensor_data_with_meta["value"].astype('float')
-
-            # print(sensor_data_with_meta["value"],,type((sensor_data_with_meta["value"].values[0])))
-
-            sensor_data_with_meta['final_value'] = (sensor_data_with_meta['value'] * sensor_data_with_meta['m']) + \
-                                                   sensor_data_with_meta['c']
-
-            # sensor_data_with_meta['final_value'] = sensor_data_with_meta['final_value'].astype('float')
-            sensor_data_with_meta['final_value'] = np.where(
-                sensor_data_with_meta["final_value"] > sensor_data_with_meta["max"], sensor_data_with_meta["max"],
-                sensor_data_with_meta["final_value"]
-            )
-            sensor_data_with_meta['final_value'] = np.where(
-                sensor_data_with_meta["final_value"] < sensor_data_with_meta["min"], sensor_data_with_meta["min"],
-                sensor_data_with_meta["final_value"]
-            )
-        df = sensor_data_with_meta[['time', 'final_value', 'sensorID']]
-        df.rename(columns={'time': 'time', 'final_value': 'value', 'sensorID': 'sensor'}, inplace=True)
-        return df, raw_metadata
+        for (value1, value2) in zip(sensor_id_list, sensor_name_list):
+            # print(value1, value2)
+            # print(data[str(value1)])
+            df_meta = pd.DataFrame(data[str(value1)])
+            # print(df_meta)
+            df_meta = df_meta.set_index('paramName').transpose()
+            if 'm' in df_meta.columns and 'c' in df_meta.columns:
+                m = int(df_meta.iloc[0]['m'])
+                c = int(df_meta.iloc[0]['c'])
+                # print(m, type(c))
+                # print(df[str(value2)])
+                df[str(value2)] = df[str(value2)].astype('float')
+                # print('==',df[str(value2)])
+                df[str(value2)] = (df[str(value2)] * m) + c
+                if 'min' in df_meta.columns:
+                    min = int(df_meta.iloc[0]['min'])
+                    df[str(value2)] = np.where(df[str(value2)] <= min, min, df[str(value2)])
+                if 'max' in df_meta.columns:
+                    max = int(str(df_meta.iloc[0]['max']).replace('-', '99999').replace(
+                        '1000000000000000000000000000', '99999').replace('100000000000', '99999'))
+                    df[str(value2)] = np.where(df[str(value2)] >= max, max, df[str(value2)])
+        return df
 
     def time_grouping(self, df, bands):
         """
@@ -173,7 +171,7 @@ class DataAccess:
 
             if response.status_code != 200:
                 raw = json.loads(response.text)
-                raise ValueError(raw)
+                raise ValueError(raw['error'])
             else:
                 raw_data = json.loads(response.text)['data']
                 df_raw = pd.DataFrame(raw_data)
@@ -183,32 +181,6 @@ class DataAccess:
             print('Failed to fetch device Details')
             print(e)
 
-    def get_device_metadata(self, device_id):
-        """
-
-        :param device_id: string
-        :return: Json
-
-         Every detail related to a particular device like device added date, m,c,min,max values, sensor id, sensor alias and so on
-
-        """
-        try:
-            url = "https://" + self.url + "/api/metaData/device/" + device_id
-            header = {'userID': self.userID}
-            payload = {}
-            response = requests.request('GET', url, headers=header, data=payload, verify=False)
-
-            if response.status_code != 200:
-                raw = json.loads(response.text)
-                raise ValueError(raw)
-            else:
-                raw_data = json.loads(response.text)['data']
-                return raw_data
-
-        except Exception as e:
-            print('Failed to fetch device Metadata')
-            print(e)
-
     def get_userinfo(self):
         """
         :return: Json
@@ -223,7 +195,7 @@ class DataAccess:
 
             if response.status_code != 200:
                 raw = json.loads(response.text)
-                raise ValueError(raw)
+                raise ValueError(raw['error'])
             else:
                 raw_data = json.loads(response.text)['data']
                 return raw_data
@@ -245,129 +217,64 @@ class DataAccess:
         Get Data Point fetches data containing values of last n data points of given sensor at given time.
 
         """
-
         try:
-            # lim = n
-            sub_list = []
-            rawdata_res = []
-            e_time = pd.to_datetime(end_time)
-            en_time = int(round(e_time.timestamp()))
-            header = {}
-            cursor = {'end': en_time,'limit': n}
-
-            if n == 1:
-                if len(sensors) == 1:
-                    url = "https://" + self.url + "/api/apiLayer/getLimitedDataMultipleSensors/?device=" + device_id + "&sensor=" + \
-                          sensors[0] + "&eTime=" + str(en_time) + "&lim=" + str(cursor['limit']) + "&cursor=true"
-                    payload = {}
-                    param = 'GET'
-                    response = requests.request(param, url, headers=header, data=payload, verify=False)
-                    raw = json.loads(response.text)
-                    if response.status_code != 200:
-                        raise ValueError(raw)
-                    if 'success' in raw:
-                        raise ValueError(raw)
-                    if len(json.loads(response.text)) == 0:
-                        raise ValueError('No Data!')
-                    else:
-                        raw_data = json.loads(response.text)['data']
-                        df_raw = pd.DataFrame(raw_data)
-                        df_raw['time'] = pd.to_datetime(df_raw['time'], utc=False)
-                        df_raw['time'] = df_raw['time'].dt.tz_convert('Asia/Kolkata')
-                        df_raw['time'] = df_raw['time'].dt.tz_localize(None)
-                        if cal or cal == 'true' or cal == "TRUE":
-                            df, metadata = DataAccess.get_caliberation(self, device_id, df_raw)
-                            df = DataAccess.get_sensor_alias(self, device_id, df, metadata)
-                        else:
-                            df = df_raw
-                            metadata = {}
-                            df = DataAccess.get_sensor_alias(self, device_id, df, metadata)
-                    return df
-
-                else:
-                    payload = {"sensors": sensors}
-                    url = "https://" + self.url + "/api/apiLayer/getLastDps?device=" + device_id
-                    param = 'PUT'
-                    response = requests.request(param, url, headers=header, data=payload, verify=False)
-                    raw = json.loads(response.text)
-                    if response.status_code != 200:
-                        raise ValueError(raw)
-                    if 'success' in raw:
-                        raise ValueError(raw)
-                    elif len(json.loads(response.text)) == 0:
-                        raise ValueError('No Data!')
-                    else:
-                        raw = json.loads(response.text)
-                        for i in raw[0]:
-                            filtered = raw[0][i]
-                            sub_list = sub_list + filtered
-                        df_raw = pd.DataFrame(sub_list)
-                        df_raw['time'] = pd.to_datetime(df_raw['time'], utc=False)
-                        df_raw['time'] = df_raw['time'].dt.tz_convert('Asia/Kolkata')
-                        df_raw['time'] = df_raw['time'].dt.tz_localize(None)
-                        if cal or cal == 'true' or cal == "TRUE":
-                            df, metadata = DataAccess.get_caliberation(self, device_id, df_raw)
-                            df = DataAccess.get_sensor_alias(self, device_id, df, metadata)
-                            df = df.sort_values('time')
-                            df = df.pivot(index='time', columns='sensor', values='value')
-                            df.reset_index(drop=False, inplace=True)
-                        else:
-                            metadata = {}
-                            df = DataAccess.get_sensor_alias(self, device_id, df_raw, metadata)
-                            df = df.sort_values('time')
-                            df = df.pivot(index='time', columns='sensor', values='value')
-                            df.reset_index(drop=False, inplace=True)
-                        return df
+            metadata = {}
+            end_time = pd.to_datetime(end_time)
+            end_time = int(round(end_time.timestamp()))
+            if type(sensors) == list:
+                len_sensors = len(sensors)
+                if len_sensors == 0:
+                    raise Exception('Message: No sensors provided')
+                n = n * len_sensors
+                delimiter = ","
+                sensor_values = delimiter.join(sensors)
             else:
-                str1 = ","
-                sensor_values = str1.join(sensors)
-                payload = {}
-                counter = 0
-                while True:
-                    for record in range(counter):
-                        sys.stdout.write('\r')
-                        sys.stdout.write("Approx Records Fetched %d" % (10000 * record))
-                        sys.stdout.flush()
-                    if counter == 0:
-                        url = "https://" + self.url + "/api/apiLayer/getLimitedDataMultipleSensors/?device=" + device_id + "&sensor=" + \
-                              sensor_values + "&eTime=" + str(en_time) + "&lim=" + str(cursor['limit']) + "&cursor=true"
-                    else:
-                        url = "https://" + self.url + "/api/apiLayer/getLimitedDataMultipleSensors/?device=" + device_id + "&sensor=" + \
-                              sensor_values + "&eTime=" + str(cursor['end']) + "&lim=" + str(cursor['limit']) + "&cursor=true"
-                    response = requests.request("GET", url, headers=header, data=payload)
-                    raw = json.loads(response.text)
-                    if response.status_code != 200:
-                        raise ValueError(raw)
-                    if 'success' in raw:
-                        raise ValueError(raw)
-                    if len(json.loads(response.text)['data']) == 0:
-                        raise ValueError('No Data!')
-                    else:
-                        raw_data = json.loads(response.text)['data']
-                        cursor = json.loads(response.text)['cursor']
-                        rawdata_res = rawdata_res + raw_data
-                        counter = counter + 1
-
-                    if cursor['end'] is None:
-                        break
-                df_raw = pd.DataFrame(rawdata_res)
-                df_raw['time'] = pd.to_datetime(df_raw['time'], utc=False)
-                df_raw['time'] = df_raw['time'].dt.tz_convert('Asia/Kolkata')
-                df_raw['time'] = df_raw['time'].dt.tz_localize(None)
-                if cal or cal == 'true' or cal == "TRUE":
-                    df, metadata = DataAccess.get_caliberation(self, device_id, df_raw)
-                    df = DataAccess.get_sensor_alias(self, device_id, df, metadata)
-                    df = DataAccess.get_cleaned_table(self, df)
+                raise Exception('Message: Incorrect type of sensors')
+            header = {}
+            cursor = {'end': end_time, 'limit': n}
+            payload = {}
+            df_raw = pd.DataFrame()
+            counter =0
+            while True:
+                for record in range(counter):
+                    sys.stdout.write('\r')
+                    sys.stdout.write("Approx Records Fetched %d" % (10000 * record))
+                    sys.stdout.flush()
+                url = "https://" + self.url + "/api/apiLayer/getLimitedDataMultipleSensors/?device=" + device_id + "&sensor=" + sensor_values + "&eTime=" + str(cursor['end']) + "&lim=" + str(cursor['limit']) + "&cursor=true"
+                response = requests.request("GET", url, headers=header, data=payload)
+                raw = json.loads(response.text)
+                if response.status_code != 200:
+                    raise ValueError(raw['error'])
+                if 'success' in raw:
+                    raise ValueError(raw['error'])
                 else:
-                    metadata = {}
-                    df = DataAccess.get_sensor_alias(self, device_id, df_raw, metadata)
-                    df = DataAccess.get_cleaned_table(self, df)
+                    raw_data = json.loads(response.text)['data']
+                    cursor = json.loads(response.text)['cursor']
+                    if len(raw_data) != 0:
+                        df_raw = pd.concat([df_raw, pd.DataFrame(raw_data)])
+                    counter = counter + 1
+                if cursor['end'] == None:
+                    break
+            if cal == True or cal == 'true' or cal == "TRUE":
+                df,metadata = DataAccess.get_sensor_alias(self, device_id, df_raw,metadata)
+                df = DataAccess.get_cleaned_table(self, df)
+                df = DataAccess.get_caliberation(self, device_id, metadata,df)
+            else:
+                df,metadata = DataAccess.get_sensor_alias(self, device_id,df_raw,metadata)
+                df = DataAccess.get_cleaned_table(self, df)
             return df
         except Exception as e:
             print(e)
 
     def data_query(self, device_id, start_time, end_time=datetime.now(), sensors=None, cal=True, bands=None, echo=True):
         # df = pd.DataFrame()
+        metadata = {}
+        if sensors == None:
+            metadata = DataAccess.get_device_metadata(self, device_id)
+            data_sensor = metadata['sensors']
+            df_sensor = pd.DataFrame(data_sensor)
+            sensor_id_list = list(df_sensor['sensorId'])
+            sensors = sensor_id_list
         rawdata_res = []
         temp = ''
         s_time = pd.to_datetime(start_time)
@@ -387,54 +294,56 @@ class DataAccess:
             if sensors is None:
                 url_api = "https://" + self.url + "/api/apiLayer/getDataByStEt?device="
                 if counter == 0:
-                    temp = url_api + device_id + "&sTime=" + str(st_time) + "&eTime=" + str(en_time) + "&cursor=true"
+                    temp = url_api + device_id + "&sTime=" + str(st_time) + "&eTime=" + str(en_time) + "&cursor=true&limit=50000"
                 else:
                     temp = url_api + device_id + "&sTime=" + str(cursor['start']) + "&eTime=" + str(
-                        cursor['end']) + "&cursor=true"
+                        cursor['end']) + "&cursor=true&limit=50000"
             if sensors is not None:
                 url_api = "https://" + self.url + "/api/apiLayer/getAllData?device="
                 if counter == 0:
                     str1 = ","
                     sensor_values = str1.join(sensors)
                     temp = url_api + device_id + "&sensor=" + sensor_values + "&sTime=" + str(st_time) + "&eTime=" + str(
-                        en_time) + "&cursor=true"
+                        en_time) + "&cursor=true&limit=50000"
                 else:
                     str1 = ","
                     sensor_values = str1.join(sensors)
                     temp = url_api + device_id + "&sensor=" + sensor_values + "&sTime=" + str(
-                        cursor['start']) + "&eTime=" + str(cursor['end']) + "&cursor=true"
+                        cursor['start']) + "&eTime=" + str(cursor['end']) + "&cursor=true&limit=50000"
 
             response = requests.request("GET", temp, headers=header, data=payload)
             raw = json.loads(response.text)
+            # print(raw)
             if response.status_code != 200:
-                raise ValueError(raw)
+                raise ValueError(raw['error'])
             if 'success' in raw:
-                raise ValueError(raw)
-            if len(json.loads(response.text)['data']) == 0:
-                raise ValueError('No Data!')
+                raise ValueError(raw['error'])
+
             else:
                 raw_data = json.loads(response.text)['data']
+                # print('====================',raw_data)
                 cursor = json.loads(response.text)['cursor']
-                rawdata_res = rawdata_res + raw_data
+                if len(raw_data) !=0:
+                    rawdata_res = rawdata_res + raw_data
                 counter = counter + 1
             if cursor['start'] is None or cursor['end'] is None:
                 break
+        # print(rawdata_res)
         df_raw = pd.DataFrame(rawdata_res)
         df_raw['time'] = pd.to_datetime(df_raw['time'], utc=False)
         df_raw['time'] = df_raw['time'].dt.tz_convert('Asia/Kolkata')
         df_raw['time'] = df_raw['time'].dt.tz_localize(None)
-
         if len(df_raw.columns) == 2:
             df_raw['sensor'] = sensors[0]
 
-        if cal == True or cal == 'true' or cal == "TRUE":
+        # print(df_raw)
+        if cal or cal == 'true' or cal == "TRUE":
             # #qq=qq[~qq['value'].isin(['','BAD 255','BAD 0','BAD undefined','-'])]
-            df, metadata = DataAccess.get_caliberation(self, device_id, df_raw)
-            df = DataAccess.get_sensor_alias(self, device_id, df, metadata)
+            df,metadata = DataAccess.get_sensor_alias(self, device_id, df_raw,metadata)
             df = DataAccess.get_cleaned_table(self, df)
+            df = DataAccess.get_caliberation(self, device_id, metadata, df)
         else:
-            metadata = {}
-            df = DataAccess.get_sensor_alias(self, device_id, df_raw, metadata)
+            df,metadata = DataAccess.get_sensor_alias(self, device_id, df_raw,metadata)
             df = DataAccess.get_cleaned_table(self, df)
         if bands is not None:
             df = DataAccess.time_grouping(self, df, bands)
@@ -470,7 +379,7 @@ class DataAccess:
 
             if response.status_code != 200:
                 raw = json.loads(response.text)
-                raise ValueError(raw)
+                raise ValueError(raw['error'])
             else:
                 raw_data = json.loads(response.text)['data']
                 return raw_data
@@ -502,7 +411,7 @@ class DataAccess:
 
             if response.status_code != 200:
                 raw = json.loads(response.text)
-                raise ValueError(raw)
+                raise ValueError(raw['error'])
             else:
                 raw_data = json.loads(response.text)['data']
                 return raw_data
@@ -532,7 +441,7 @@ class DataAccess:
 
             if response.status_code != 200:
                 raw = json.loads(response.text)
-                raise ValueError(raw)
+                raise ValueError(raw['error'])
             else:
                 raw_data = json.loads(response.text)
                 return raw_data
@@ -557,7 +466,7 @@ class DataAccess:
 
             if response.status_code != 200:
                 raw = json.loads(response.text)
-                raise ValueError(raw)
+                raise ValueError(raw['error'])
             else:
                 raw_data = json.loads(response.text)['data']
                 return raw_data
